// returns +1 on clock-wise, -1 on couter-clock-wise

#include <pic18f252.h>

#define ENC1 PORTCbits.RC0
#define ENC2 PORTCbits.RC1
#define ENCB PORTAbits.RA4

extern void encoder_button();
extern void encoder_left();
extern void encoder_right();

signed char read_encoder() {
    static char buff;

    buff <<= 1;
    buff += ENC1;
    buff <<= 1;
    buff += ENC2;

    switch (buff & 0x0F) {
        case 0b00000001: return 1;
        case 0b00000010: return -1;
        case 0b00000100: return -1;
        case 0b00000111: return 1;
        case 0b00001000: return 1;
        case 0b00001011: return -1;
        case 0b00001101: return -1;
        case 0b00001110: return 1;
    }
    return 0;
}

void run_encoder() {
    static signed char enc;
    static unsigned char encb;

    // read the encoder button - debounce:
    if (ENCB)
        encb = 0;
    else
        if (encb < 0xFE) encb++;

    if (encb == 0xFE) {
        encoder_button();
        encb++;
    }

    enc += read_encoder();
    if (enc > 3) {
        enc = 0;
        encoder_left();
    }
    if (enc < -3) {
        enc = 0;
        encoder_right();
    }
}

/*
Each sample gives you two bits. You combine these two bits with the two bits you got from the previous sample to get 4 bits. Use these 4 bits in a lookup table to count up, count down, or do nothing:

0 0 0 0 - do nothing
0 0 0 1 - count up
0 0 1 0 - count down
0 0 1 1 - do nothing (*)
0 1 0 0 - count down
0 1 0 1 - do nothing
0 1 1 0 - do nothing (*)
0 1 1 1 - count up
1 0 0 0 - count up
1 0 0 1 - do nothing (*)
1 0 1 0 - do nothing
1 0 1 1 - count down
1 1 0 0 - do nothing (*)
1 1 0 1 - count down
1 1 1 0 - count up
1 1 1 1 - do nothing

The first two bits are the old A and B phases and the last two bits are the new A and B phases. The "do nothing(*)" cases are ones where something went wrong.
 */