/* 
 * File:   main.c
 * Author: k15
 *
 * Created on 5. srpen 2015, 14:21
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>

// PIC18F252 Configuration Bit Settings
// 'C' source line config statements

#include <xc.h>

#include "display.h"
#include "encoder.h"
#include "thermometer.h"

// #pragma config statements should precede project file includes.
// Use project enums instead of #define for ON and OFF.

// CONFIG1H
#pragma config OSC = HS         // Oscillator Selection bits (HS oscillator)
#pragma config OSCS = OFF       // Oscillator System Clock Switch Enable bit (Oscillator system clock switch option is disabled (main oscillator is source))

// CONFIG2L
#pragma config PWRT = OFF       // Power-up Timer Enable bit (PWRT disabled)
#pragma config BOR = ON         // Brown-out Reset Enable bit (Brown-out Reset enabled)
#pragma config BORV = 20        // Brown-out Reset Voltage bits (VBOR set to 2.0V)

// CONFIG2H
#pragma config WDT = OFF        // Watchdog Timer Enable bit (WDT disabled (control is placed on the SWDTEN bit))
#pragma config WDTPS = 128      // Watchdog Timer Postscale Select bits (1:128)

// CONFIG3H
#pragma config CCP2MUX = ON     // CCP2 Mux bit (CCP2 input/output is multiplexed with RC1)

// CONFIG4L
#pragma config STVR = ON        // Stack Full/Underflow Reset Enable bit (Stack Full/Underflow will cause RESET)
#pragma config LVP = OFF        // Low Voltage ICSP Enable bit (Low Voltage ICSP enabled)

// CONFIG5L
#pragma config CP0 = OFF        // Code Protection bit (Block 0 (000200-001FFFh) not code protected)
#pragma config CP1 = OFF        // Code Protection bit (Block 1 (002000-003FFFh) not code protected)
#pragma config CP2 = OFF        // Code Protection bit (Block 2 (004000-005FFFh) not code protected)
#pragma config CP3 = OFF        // Code Protection bit (Block 3 (006000-007FFFh) not code protected)

// CONFIG5H
#pragma config CPB = OFF        // Boot Block Code Protection bit (Boot Block (000000-0001FFh) not code protected)
#pragma config CPD = OFF        // Data EEPROM Code Protection bit (Data EEPROM not code protected)

// CONFIG6L
#pragma config WRT0 = OFF       // Write Protection bit (Block 0 (000200-001FFFh) not write protected)
#pragma config WRT1 = OFF       // Write Protection bit (Block 1 (002000-003FFFh) not write protected)
#pragma config WRT2 = OFF       // Write Protection bit (Block 2 (004000-005FFFh) not write protected)
#pragma config WRT3 = OFF       // Write Protection bit (Block 3 (006000-007FFFh) not write protected)

// CONFIG6H
#pragma config WRTC = OFF       // Configuration Register Write Protection bit (Configuration registers (300000-3000FFh) not write protected)
#pragma config WRTB = OFF       // Boot Block Write Protection bit (Boot Block (000000-0001FFh) not write protected)
#pragma config WRTD = OFF       // Data EEPROM Write Protection bit (Data EEPROM not write protected)

// CONFIG7L
#pragma config EBTR0 = OFF      // Table Read Protection bit (Block 0 (000200-001FFFh) not protected from Table Reads executed in other blocks)
#pragma config EBTR1 = OFF      // Table Read Protection bit (Block 1 (002000-003FFFh) not protected from Table Reads executed in other blocks)
#pragma config EBTR2 = OFF      // Table Read Protection bit (Block 2 (004000-005FFFh) not protected from Table Reads executed in other blocks)
#pragma config EBTR3 = OFF      // Table Read Protection bit (Block 3 (006000-007FFFh) not protected from Table Reads executed in other blocks)

// CONFIG7H
#pragma config EBTRB = OFF      // Boot Block Table Read Protection bit (Boot Block (000000-0001FFh) not protected from Table Reads executed in other blocks)

/*
 * 
 */

unsigned char power;
unsigned char current_temperature;
unsigned char preset_temperature;

#define PWR_OUT1 LATAbits.LATA5
#define PWR_OUT23 LATAbits.LATA3
#define DISPLAY_SELECT LATAbits.LATA1

bool flash;
bool heat;

enum {
    run,
    set_temperature,
    set_plates
} mode;

enum {
    plate1,
    plate23,
    plate123
} plates;

void init() {
    TRISA = 0xD1;
    TRISB = 0x00;
    TRISC = 0x03;

    T0CON = 0b10001000;
    ADCON0 = 0b10000001;
    ADCON1 = 0b11001110;

    TMR0IE = 1;
    GIE = 1;

    power = 100;
    preset_temperature = 100;
    mode = run;
}

int main(int argc, char** argv) {

    init();

    while (1) {
        run_encoder();
        CLRWDT();
    }

    return (EXIT_SUCCESS);
}

void display_plates() {
    switch (plates) {
        case plate1:
            display_value(1);
            break;
        case plate23:
            display_value(23);
            break;
        case plate123:
            display_value(123);
            break;
    }
}

void display_values() {
    display_off();
    DISPLAY_SELECT = !DISPLAY_SELECT;
    if (DISPLAY_SELECT) {
        if (mode == set_temperature) {
            if (flash)
                display_value(preset_temperature);
        } else {
            display_value(current_temperature);
        }
    } else {
        if (mode == set_plates) {
            if (flash)
                display_plates();
        } else {
            if (heat || flash)
                display_value(power);
        }
    }
}

void i200ms() {
    static unsigned char sample100;
    static unsigned char sample5;

    if (sample5 > 2) {
        sample5 = 0;
        flash = !flash;
    }
    sample5++;

    if (sample100 > 99) sample100 = 0;
    if (sample100 < power && heat) {
        PWR_OUT1 = plates != plate23;
        PWR_OUT23 = plates != plate1;
    } else {
        PWR_OUT1 = 0;
        PWR_OUT23 = 0;
    }
    sample100++;

    GODONE = 1;
    while (GODONE);
    current_temperature = value_to_temperature(ADRES);
    if (current_temperature == 0xFF) {
        heat = true;
    } else {
        if (heat) {
            heat = current_temperature < preset_temperature;
        } else {
            heat = current_temperature < preset_temperature - 1;
        }
    }
}

void interrupt i1ms() {

    static unsigned char count200;

    if (count200-- == 0) {
        i200ms();
        count200 = 200;
    }

    display_values();

    TMR0 = 62535;
    TMR0IF = 0;
}

void encoder_button() {
    mode++;
    if (mode > set_plates)
        mode = run;
}

void encoder_left() {
    switch (mode) {
        case run:
            if (power < 100)
                power += 5;
            break;
        case set_temperature:
            if (preset_temperature < 150)
                preset_temperature++;
            break;
        case set_plates:
            if (plates < plate123)
                plates++;
            break;
    }
}

void encoder_right() {
    switch (mode) {
        case run:
            if (power > 0)
                power -= 5;
            break;
        case set_temperature:
            if (preset_temperature > 0)
                preset_temperature--;
            break;
        case set_plates:
            if (plates > plate1)
                plates--;
            break;
    }
}


